/*
 * AD7734.c
 *
 * Created: 13.12.2014 16:14:22
 *  Author: Moataz
 */ 



#include "AD7734.h"


unsigned char AD7734_Ready(char ch)
{
	unsigned char rdy = 0;
	ch--;
	rdy = (AD7734_GetRegisterValue( AD7734_REG_STATUS,1) & (1 << ch));
	
	return(rdy);
}


uint8_t AD7734_Init(void)
{
	uint8_t status_u8 = 0x1;
	if((AD7734_GetRegisterValue(AD7734_REG_REV, 1) & 0x0F) != 0x2)
	{
		status_u8 = 0x0;
	}
	
	return(status_u8);
}

void AD7734_Reset(void)
{
	unsigned char dataToSend[4] = {0xff, 0xff, 0xff, 0xff};
	spiWriteData(dataToSend,4);
	PORTB &= ~(1<<AD7734_RESET);
	_delay_ms(5);
	PORTB |= (1<<AD7734_RESET);
}


void AD7734_Setup(void)
{
	volatile uint8_t channelSetup_u8 = 0;
	volatile unsigned long Value = 0;
	// Continuous conversion on channel 1, enable 24 bits mode
	/*
	AD7734_SetRegisterValue( AD7734_REG_CH1_MODE,
	AD7734_MODE_MD(1) |
	AD7734_MODE_24BIT,
	1
	);
	*/
	channelSetup_u8 = AD7734_CHSETUP_ENABLE | AD7734_CHRNG0_ENABLE;
	// Enable Channel 1, Nominal input voltage +-10V
	AD7734_SetRegisterValue( AD7734_REG_CH1_SETUP,
	channelSetup_u8,
	1
	);
	
	// Set channel conversion time
	AD7734_SetRegisterValue( AD7734_REG_CH1_CONVTIME,
	10,
	1
	);
	channelSetup_u8 = AD7734_GetRegisterValue( AD7734_REG_CH1_CONVTIME,1);
	// Single conversion on channel 1, enable 24 bits mode
	AD7734_SetRegisterValue( AD7734_REG_CH1_MODE,
	AD7734_MODE_MD(2) |
	AD7734_MODE_24BIT ,
	1
	);
	//channelSetup_u8 = AD7734_GetRegisterValue( AD7734_REG_CH1_MODE,1);
	
	
	channelSetup_u8 = AD7734_GetRegisterValue( AD7734_REG_CH1_SETUP,1);
	
	//channelSetup_u8 = AD7734_GetRegisterValue( AD7734_REG_CH1_MODE,1);
	Value = AD7734_GetRegisterValue( AD7734_REG_CH1_DATA,3);
	
	
	
	
	// Enable Channel 2, Nominal input voltage +-10V
	AD7734_SetRegisterValue( AD7734_REG_CH2_SETUP,
	channelSetup_u8,
	1
	);
	// Set channel conversion time
	AD7734_SetRegisterValue( AD7734_REG_CH2_CONVTIME,
	17,
	1
	);

	
}


unsigned long AD7734_GetRegisterValue(uint8_t regAddress_u8, uint8_t size_u8)
{
	uint8_t data_u8a[5] = {0x03, 0x00, 0x00, 0x00, 0x00};
	unsigned long receivedData = 0x00;
	data_u8a[0] = AD7734_COMM_READ |  AD7734_COMM_ADDR(regAddress_u8);
	PORTA &= ~(1<<AD7734_CS);
	spiWriteData(data_u8a,1);
	PORTA |= (1<<AD7734_CS);
	_delay_ms(5);
	PORTA &= ~(1<<AD7734_CS);
	spiReadData(data_u8a,size_u8);
	PORTA |= (1<<AD7734_CS);
	if(size_u8 == 1)
	{
		receivedData += (data_u8a[0] << 0);
	}
	if(size_u8 == 2)
	{
		receivedData += (data_u8a[0] << 8);
		receivedData += (data_u8a[1] << 0);
	}
	if(size_u8 == 3)
	{
		receivedData += ((unsigned long)data_u8a[0] << 16);
		receivedData += (data_u8a[1] << 8);
		receivedData += (data_u8a[2] << 0);
	}
	
	return receivedData;
}


void AD7734_SetRegisterValue(uint8_t regAddress_u8, unsigned long regValue_u32, uint8_t size_u8)
{
	uint8_t data_u8a[5] = {0x03, 0x00, 0x00, 0x00, 0x00};
	
	data_u8a[0] = AD7734_COMM_WRITE |  AD7734_COMM_ADDR(regAddress_u8);
	if(size_u8 == 1)
	{
		data_u8a[1] = (unsigned char)regValue_u32;
	}
	if(size_u8 == 2)
	{
		data_u8a[2] = (unsigned char)((regValue_u32 & 0x0000FF) >> 0);
		data_u8a[1] = (unsigned char)((regValue_u32 & 0x00FF00) >> 8);
	}
	if(size_u8 == 3)
	{
		data_u8a[3] = (unsigned char)((regValue_u32 & 0x0000FF) >> 0);
		data_u8a[2] = (unsigned char)((regValue_u32 & 0x00FF00) >> 8);
		data_u8a[1] = (unsigned char)((regValue_u32 & 0xFF0000) >> 16);
	}
	PORTA &= ~(1<<AD7734_CS);
	spiWriteData(data_u8a,size_u8+1);
	PORTA |= (1<<AD7734_CS);
}